from __future__ import annotations

from tcsoa.gen.Core._2009_10.DataManagement import GetTablePropertiesResponse, GetItemFromAttributeResponse, TableInfo, GetItemFromAttributeInfo
from tcsoa.gen.Core._2007_01.DataManagement import GetItemFromIdPref
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.BusinessObjects import Table
from tcsoa.base import TcService
from tcsoa.gen.Core._2009_10.ProjectLevelSecurity import UserProjectsInfoResponse, UserProjectsInfoInput


class DataManagementService(TcService):

    @classmethod
    def getTableProperties(cls, table: List[Table]) -> GetTablePropertiesResponse:
        """
        This operation allows client applications to obtain the properties pertaining to one or more Table Business
        Objects. Client developers will need to pass in references to each Table that they need to query information
        on.  Note that the input vector needs to contain only references to the Teamcenter Table business object, this
        operation cannot be used to fetch properties of any other Business Objects.
        
        Use cases:
        This operation can be used by a client developer, when he/she deals with obtaining specific Table Business
        Object specific properties. Typically, Table Business Objects are not themselves visible in the Teamcenter
        workspace and appear as properties of other owning objects that are visible in the workspace.  The typical
        scenario in such cases is that a user attempts to obtain/view all properties of the the owning object, which
        may have one or more reference properties pointing to a Table.  A custom UI would need to display the Table
        related properties on the parent object in such cases and rendering these properties would require the client
        applications to obtain such information using the getTableProperties operation.
        One such example of existing usage of this operation, is the existing Teamcenter Rich Client functionality for
        viewing properties an Integer type of Parameter Definition [ ParmDefIntRevision Business Object ]. This
        Business Object and the functionality for viewing its properties are provided by the add on Calibration and
        Configuration Data Management module, through custom stylesheets which render a Table like UI for each
        referenced Table property.  At the time of rendering the UI, the client operations call the getTableProperties
        operation to obtain properties such as the number of rows, number of columns, labels for each row and column,
        the type of the cells and cell values and descriptions for each cell in the table.
        """
        return cls.execute_soa_method(
            method_name='getTableProperties',
            library='Core',
            service_date='2009_10',
            service_name='DataManagement',
            params={'table': table},
            response_cls=GetTablePropertiesResponse,
        )

    @classmethod
    def setTableProperties(cls, tableData: List[TableInfo]) -> ServiceData:
        """
        This operation allows client applications to set the properties pertaining to one or more Table business
        objects. Client developers will need to set information pertaining to the number of rows, columns, descriptions
        of each row and column, and cell information for each cell of the Table. The cell information must contain the
        type of cell, value to be placed in the cell, and optionally, a description of those values. The current
        operation only works on cells of specific types and it is mandatory that the type of the cells being set on the
        input structure corresponds to one of the cell types defined in the database schema viewable through the BMIDE.
        Supported valid types are:
        - TableCellInt
        - TableCellString
        - TableCellDouble
        - TableCellLogical
        - TableCellHex
        - TableCellSED
        - TableCellBCD
        - TableCellDate
        
        
        
        Use cases:
        This operation can be used by a client developer, when he/she deals with setting Table Business Object specific
        properties. Typically, Table Business Objects are not themselves visible in the Teamcenter workspace and appear
        as properties of other owning objects that are visible in the workspace. Modification to the owning objects,
        may involve changes to one or more of the Table properties that they reference through the Table.  In such
        cases, the setTableProperties is to be called, passing in the input structure which is setup to specify the
        modified values.
        One such example of existing usage of this operation, is the existing Teamcenter Rich Client functionality for
        modification of an Integer type of Parameter Definition [ ParmDefIntRevision Business Object ]. This Business
        Object and the functionality for its modification are provided by the add on Calibration and Configuration Data
        Management module.  During modification, of the integer parameter definition object, client code renders table
        like UI for each table property of the Parameter Definition, gathers the input values from the UI and populates
        a vector of the input structure of type TableInfo, sets the type of thecells to TableCellInt and makes the
        operation call. Client code will then parse the Service Data returned from the operation to obtain a handle to
        the updated Table Business Object. Errors, if any were encountered during the operation execution, are handled
        via the Service Data.
        """
        return cls.execute_soa_method(
            method_name='setTableProperties',
            library='Core',
            service_date='2009_10',
            service_name='DataManagement',
            params={'tableData': tableData},
            response_cls=ServiceData,
        )

    @classmethod
    def getItemFromAttribute(cls, infos: List[GetItemFromAttributeInfo], nRev: int, pref: GetItemFromIdPref) -> GetItemFromAttributeResponse:
        """
        This service retrieves Item and its related ItemRevision objects based on the supplied attribute key-value
        pairs supplied in the 'infos' list. All the key-value pairs except for the 'rev_id' key are used to create a
        query for Item objects.
        
        Once a set of Item objects have been retrieved, their ItemRevision objects are retrieved based on the following
        rules:
        - If  'nRev' is a negative value then all the ItemRevision objects are returned
        - If 'nRev' is a positive value then the 'nRev' most recent ItemRevision objects are returned. If 'nRev' is
        greater than the number of revisions then all of them are returned.
        - If 'nRev' is zero and a 'rev_id' attribute key was supplied in the attribute key-value pairs, then that
        ItemRevision object is returned.
        - If 'nRev' is zero and rev ids values were supplied in the 'GetItemFromAttributeInfo' object then all of the
        specified rev ids will be returned.
        
        """
        return cls.execute_soa_method(
            method_name='getItemFromAttribute',
            library='Core',
            service_date='2009_10',
            service_name='DataManagement',
            params={'infos': infos, 'nRev': nRev, 'pref': pref},
            response_cls=GetItemFromAttributeResponse,
        )


class ProjectLevelSecurityService(TcService):

    @classmethod
    def getUserProjects(cls, userProjectsInfoInputs: List[UserProjectsInfoInput]) -> UserProjectsInfoResponse:
        """
        This operation returns the list of  TC_Project objects for each of  the users in the input structure based on
        the additional criteria like active projects only, user privileged projects only and programs only. The output
        for this operation is a UserProjectsInfoResponse structure.
        """
        return cls.execute_soa_method(
            method_name='getUserProjects',
            library='Core',
            service_date='2009_10',
            service_name='ProjectLevelSecurity',
            params={'userProjectsInfoInputs': userProjectsInfoInputs},
            response_cls=UserProjectsInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class InternalContext(TcBaseObj):
    """
    The InternalContext structure represents the preference context values.
    
    :var contextName: Context name for the values.
    The default one is "Teamcenter". It can also be set to be used with NXManager, by setting the 'TC_Application'
    environment variable to UGMAN.
    
    :var value: List of values applicable for the context.
    """
    contextName: str = ''
    value: List[str] = ()


@dataclass
class InternalPreferenceDefinition(TcBaseObj):
    """
    Represents the definition for a preference.
    
    :var preferenceName: Name of the preference.
    :var preferenceCategory: Category of the preference.
    :var preferenceDescription: Description of the preference.
    :var preferenceScope: Scope of the preference.
    Valid values are "USER", "ROLE", "GROUP" or "SITE".
    :var preferenceType: Type of the preference.
    Valid values are "STRING", "LOGICAL", "INTEGER", "DOUBLE" or "DATE".
    :var isArray: Specifies if this preference takes multiple values.
    :var isDisabled: Specifies if this preference is enabled.
    """
    preferenceName: str = ''
    preferenceCategory: str = ''
    preferenceDescription: str = ''
    preferenceScope: str = ''
    preferenceType: str = ''
    isArray: bool = False
    isDisabled: bool = False


@dataclass
class InternalPreferenceInfo(TcBaseObj):
    """
    Contains the entire information about a preference: the definition is captured in the PreferenceDefinition
    structure and the values in the list of InternalContext structures.
    
    :var preferenceDefinition: The definition information.
    :var contextInformation: The contextual values.
    """
    preferenceDefinition: InternalPreferenceDefinition = None
    contextInformation: List[InternalContext] = ()


@dataclass
class InternalPreferencesInfo(TcBaseObj):
    """
    A list of InternalPreferenceInfo structures.
    
    :var prefsInfo: List of InternalPreferenceInfo structure.
    """
    prefsInfo: List[InternalPreferenceInfo] = ()


@dataclass
class PreferenceExportResponse(TcBaseObj):
    """
    The response received after an export operation.
    
    :var fileTicket: Ticket of the exported file present in the transient volume uploaded by the server.
    :var serviceData: Holds the list of errors during the operation.
    """
    fileTicket: str = ''
    serviceData: ServiceData = None


@dataclass
class PreferenceResponse(TcBaseObj):
    """
    Represents the preferences information.
    
    :var preferencesArray: List of preferences information, which contains preference definitions and values.
    :var serviceData: Holds the list of errors during the operation.
    """
    preferencesArray: List[InternalPreferencesInfo] = ()
    serviceData: ServiceData = None


@dataclass
class PreferencesDeleteInput(TcBaseObj):
    """
    The PreferencesDeleteInput structure represents the information about a preference that
    will be passed as input to the delete method.
    
    :var preferenceScope: Scope of the preference. Scope can be "USER", "ROLE", "GROUP" or "SITE".
    :var preferenceName: Name of the preference.
    :var object: Object to which the preferences have to be assigned. This can be
    a User, Role or Group. If no value is specified, preferences from the
    current session are considered based on the scope. To be able to delete
    preferences for the site, the user must be a DBA. To be able to delete
    preferences for the role and site, the user must be a GA or DBA.
    """
    preferenceScope: str = ''
    preferenceName: str = ''
    object: BusinessObject = None


@dataclass
class PreferencesExportInput(TcBaseObj):
    """
    The input required to export preferences.
    
    :var preferenceScope: Location (as seen from the current logged-in user) from where to export preferences.
    Valid values are "USER", "ROLE", "GROUP" or "SITE".
    :var categoryNames: List of category names which are to be considered by the operation.
    If no value is provided, the category filter is ignored and preferences from all categories are considered by the
    operation.
    """
    preferenceScope: str = ''
    categoryNames: List[str] = ()


@dataclass
class PreferencesImportInput(TcBaseObj):
    """
    Specifies the input to be considered for the import operation.
    
    :var preferenceScope: Location where to import the preferences. 
    Valid values are "USER", "ROLE", "GROUP" or "SITE".
    :var fileTicket: Ticket of the input XML file in the transient volume uploaded by the client.
    :var categoryNames: List of category names which are to be considered by the operation. If no value is provided,
    the category filter is ignored and preferences from all categories are considered by the operation.
    :var importAction: Specifies what to do when the preference already exists. Can be "SKIP", "OVERRIDE", or "MERGE".
    """
    preferenceScope: str = ''
    fileTicket: str = ''
    categoryNames: List[str] = ()
    importAction: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Core._2012_10.DataManagement import TraceabilityReportOutput1, TraceabilityInfoInput1, GetDatasetTypesWithFileExtensionResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def getTraceReport(cls, input: List[TraceabilityInfoInput1]) -> TraceabilityReportOutput1:
        """
        This operation will generate a Trace Report on the objects selected by user. The report will contain
        information about complying as well as defining objects which are connected to selected object using
        FND_TraceLink, or its subtype. This operation will check if there is any TraceLink relation starting or ending
        from selected object(s). If TraceLink relation exists for selected object(s), then it gets the other end of
        TraceLink relation and generates a trace report.  
        
        Trace links can be between following objects: 
        
        * Between occurrences of an ItemRevision 
        * Between any two WorkspaceObject. 
        
        Following will be added, in addition to existing getTraceReport operation: 
        
        * If scope of search structure is defined for the getting Trace Report in input of this operation by sending
        top lines of BOMWindow instances, then matching TraceLink instances within the scope windows will be returned. 
        * If input of this operation is having list of object type names, then object type filter will be applied to
        target objects of TraceLink. 
        * If input of this operation is having list of TraceLink type names, then those types of TraceLink will be
        returned in Trace Report. 
        * If propertyFilterInput is given in the input of this operation, then the additional filter of property will
        be applied on the output before sending to client. 
        * Trace report tree will be sorted for given property, sort direction can also be defined, if not defined then
        it will get default sorted in ascending direction. 
        * The output of this operation can be either sent to rich client to build the report or to MSExcel application. 
        * User can export this trace report to MSExcel application by sending appropriate exportTo mode in input. If
        the mode of export is "TraceReportMSExcelExport", then trace report will be exported to .xlsm file and this
        file ticket will be sent to rich client. Then rich client will download the file and open MSExcel application. 
        """
        return cls.execute_soa_method(
            method_name='getTraceReport',
            library='Core',
            service_date='2012_10',
            service_name='DataManagement',
            params={'input': input},
            response_cls=TraceabilityReportOutput1,
        )

    @classmethod
    def refreshObjects2(cls, objects: List[BusinessObject], lockObjects: bool) -> ServiceData:
        """
        This operation is used to reload the in-memory representation of the objects from the database.
        Any references to the object will still be valid. Any in-memory changes to the original object will be lost. If
        the object has been changed in the database since it was last loaded, then those changes will not be present in
        memory. 
        The operation updates the in memory representation to reflect database changes. 
        
        If the lockObjects flag is true then it will aquire write lock on objects otherwise operation will release the
        write lock on the business objects. 
        This is useful when client needs to do an in-process lock and unlock for shorter duration that does not require
        checkout or checkin mechanism. Client caling this operation to lock the objects must unlock those objects by
        callng this operation.
        This operation must be used in pairs to lock and unlock the objects.
        
        Use cases:
        Use this operation to do bulk lock & bulk unlock of Input Objects.
        """
        return cls.execute_soa_method(
            method_name='refreshObjects2',
            library='Core',
            service_date='2012_10',
            service_name='DataManagement',
            params={'objects': objects, 'lockObjects': lockObjects},
            response_cls=ServiceData,
        )

    @classmethod
    def getDatasetTypesWithFileExtension(cls, fileExtensions: List[str]) -> GetDatasetTypesWithFileExtensionResponse:
        """
        This operation returns the dataset type and reference information for a set of file extensions. Named
        references are Teamcenter objects that relate to a specific data file. For each file extension, it is possible
        that it belongs to multiple dataset types. For such cases, all matching dataset types will be returned using
        the file extension as the key in the GetDatasetTypesWithFileExtensionOutput structure. The order of file
        extension in the GetDatasetTypesWithFileExtensionOutput structure may be different than the order of file
        extension input. This operation will insert file extensions that match the default dataset type defined in
        AE_default_dataset_type preference at the beginning of the list. This operation uses
        TC_Dataset_Import_Exclude_Wildcard preference to determine if wildcard may be used in file extension input. If
        the preference is set and file extension is set to asterisk, this operation will return all data set types that
        allow wildcards in its name reference in Teamcenter. Details about these two preferences can be found in
        Preferences and Environment (Variables Reference Configuration preferences, under Data management preferences).
        """
        return cls.execute_soa_method(
            method_name='getDatasetTypesWithFileExtension',
            library='Core',
            service_date='2012_10',
            service_name='DataManagement',
            params={'fileExtensions': fileExtensions},
            response_cls=GetDatasetTypesWithFileExtensionResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject, ItemRevision
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ExtendedAttributes(TcBaseObj):
    """
    Structure to collect additional attributes to be assigned apart from default 'Save As' or 'Cretae New' Item
    operation.
    
    :var objectType: Type of Business Object.
    :var attributes: Map of attribute name and its value. pairs( String, String )
    """
    objectType: str = ''
    attributes: AttributeNameValueMap = None


@dataclass
class ItemProperties(TcBaseObj):
    """
    Specifies attributes for the new Item or ItemRevision. Mainly used in 'Create new Item' scenario, describes
    specific attributes to be assigned.
    
    :var clientId: Identifier that helps the client to track the object(s) created
    :var itemId: Id of the Item to be created
    :var name: Name of the Item to be created
    :var type: Business Object type of the Item to be created
    :var revId: Id of the initial revision of the Item to be created
    :var uom: UOM of the Item to be created
    :var description: Description of the Item to be created
    :var extendedAttributes: List of ''extendedAttributes'' structures having Name/value pairs for additional
    attributes to set.
    """
    clientId: str = ''
    itemId: str = ''
    name: str = ''
    type: str = ''
    revId: str = ''
    uom: str = ''
    description: str = ''
    extendedAttributes: List[ExtendedAttributes] = ()


@dataclass
class SaveAsIn(TcBaseObj):
    """
    Input for save as operation
    
    :var targetObject: Target object being saved as
    :var saveAsInput: SaveAsInput (user input from SaveAs dialog) for the targetobject being saved as
    :var deepCopyDatas: DeepCopyData of the objects attached to the targetobject
    """
    targetObject: BusinessObject = None
    saveAsInput: SaveAsInput = None
    deepCopyDatas: List[DeepCopyData] = ()


@dataclass
class SaveAsInput(TcBaseObj):
    """
    Structure encapsulating all information related to a Workspace Object which may be required for "Save As" operation
    during operations ''createVariantItem'' or ''createAndSubstituteVariantItem''.
    
    :var boName: Type Name Of Business Object
    :var stringProps: Map containing string property values . Pairs (string, string)
    :var boolArrayProps: Map containing boolean array property values. Pairs (string, boolean[])
    :var dateProps: Map containing date property values. Pairs (string, Date)
    :var dateArrayProps: Map containing date array property values . Pairs (string, Date[])
    :var tagProps: Map containing tag property values. Pairs (string, Tag)
    :var tagArrayProps: Map containing tag array property values. Pairs (string, Tag[])
    :var stringArrayProps: Map containing string array property values. Pairs (string, string[])
    :var doubleProps: Map containing double property values. Pairs (string, double)
    :var doubleArrayProps: Map containing double array property values. Pairs (string,  double[])
    :var floatProps: Map containing float property values. Pairs (string, float)
    :var floatArrayProps: Map containing float array property values. Pairs (string, float[])
    :var intProps: Map containing integer property values. Pairs (string, integer)
    :var intArrayProps: Map containing integer array property values. Pairs (string, integer[])
    :var boolProps: Map containing boolean property values. Pairs (string, boolean)
    """
    boName: str = ''
    stringProps: StringMap = None
    boolArrayProps: BoolVectorMap = None
    dateProps: DateMap = None
    dateArrayProps: DateVectorMap = None
    tagProps: TagMap = None
    tagArrayProps: TagVectorMap = None
    stringArrayProps: StringVectorMap = None
    doubleProps: DoubleMap = None
    doubleArrayProps: DoubleVectorMap = None
    floatProps: FloatMap = None
    floatArrayProps: FloatVectorMap = None
    intProps: IntMap = None
    intArrayProps: IntVectorMap = None
    boolProps: BoolMap = None


@dataclass
class CreateAndSubsVIInput(TcBaseObj):
    """
    Input structure for operation ''createAndSubstituteVariantItem''. One structure represents One new variant Item to
    be created.
    
    :var genericBOMLine: BOMLine of configured structure from which new variant Item Structure should be created. This
    structure has Variants defined and can be configured for some given variant Configuration. For more details please
    refer section 'Creating and updating variant Items' from Teamcenter Documentation.
    :var createOrSaveAsDescriptor: Create or SaveAs description required for Item Creation. This will include
    information about new variant Item creation such as Item Id, name, description, objects that need to be copied or
    done 'Save As' along with new Item.
    :var viBOMLine: BOMLine of variant Item structure. After creating new variant Item from genericBOMLine, the newly
    created variant ItemRevision will be substituted on this viBOMLine.
    :var linkVIToGenericItem: Flag to decide Variant Item should be linked to its generic structure or not. Value of
    this parameter will be overriden by preference ''PSEIsNewVILinkedToModule'', if exists on SITE level.
    :var findVIBeforeCreate: Find existing Variant Item before creating new Variant Item. If existing Variant Item is
    found, then new Variant Item is not created. This existing Variant Item will be replaced on viBOMLine.
    """
    genericBOMLine: BOMLine = None
    createOrSaveAsDescriptor: CreateOrSaveAsDescriptor = None
    viBOMLine: BOMLine = None
    linkVIToGenericItem: bool = False
    findVIBeforeCreate: bool = False


@dataclass
class CreateAndSubsVIOutput(TcBaseObj):
    """
    Output structure for 'CreateAndSubstituteVI' SOA
    
    :var genericBOMLine: Generic BOMLine from which variant Item is created.
    :var viBOMLine: BOMLine of variant Item structure. After creating new variant Item from 'genericBOMLine', the newly
    created variant ItemRevision will be substituted on this 'viBOMLine'.
    :var newVariantItemRevision: ItemRevision of newly created Variant Item.
    :var isExistingVIFound: This flag will be returned populated only when flag ''findVIBeforeCreate'' in server input
    ''CreateAndSubsVIInput'' is passed as 'True'. Flag to indicate that if existing Variant Item was not found and new
    Variant Item is created. If value is true, this means existing Variant Item was found, and was used for substitute
    operation.
    """
    genericBOMLine: BOMLine = None
    viBOMLine: BOMLine = None
    newVariantItemRevision: ItemRevision = None
    isExistingVIFound: bool = False


@dataclass
class CreateAndSubsVIResponse(TcBaseObj):
    """
    Response structure to operation ''createAndSubstituteVariantItem''.
    
    :var createAndSubsOutputs: List of 'CreateAndSubsVIOutput' s for each 'CreateAndSubsVIInput'.
    :var serviceData: Service Data to return error back to caller.
    """
    createAndSubsOutputs: List[CreateAndSubsVIOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrSaveAsDescriptor(TcBaseObj):
    """
    This is helper structure to collect all information of new variant Item required for either creating New Item or
    saving existing Item as variant Item.
    
    :var properties: Input properties for new variant Item.
    :var saveAsIn: Input for save as operation
    :var bCreateOrSaveAsFlag: bool flag to indicate Create new Item as true and "Save as" operation as false value.
    """
    properties: List[ItemProperties] = ()
    saveAsIn: SaveAsIn = None
    bCreateOrSaveAsFlag: bool = False


@dataclass
class CreateVIInput(TcBaseObj):
    """
    Input Structure for operation ''createVariantItem''.
    
    :var genericBOMLine: BOMLine of configured structure from which new variant Item Structure should be created. This
    structure has Variants defined and can be configured for some given variant Configuration. For more details please
    refer section 'Creating and updating variant Items' from Teamcenter Documentation.
    :var createOrSaveAsDescriptor: Create or SaveAs description required for Item Creation. This will include
    information about new variant Item creation such as Item Id, name, description, objects that need to be copied or
    done 'Save As' along with new Item.
    :var linkVIToGenericItem: Flag to decide variant Item should be linked to its generic structure or not. Value of
    this parameter will be overridden by preference 'PSEIsNewVILinkedToModule', if exists on SITE location.
    """
    genericBOMLine: BOMLine = None
    createOrSaveAsDescriptor: CreateOrSaveAsDescriptor = None
    linkVIToGenericItem: bool = False


@dataclass
class CreateVIOutput(TcBaseObj):
    """
    Output structure for operation ''createVariantItem''. There will be one 'CreateVIOutput' for each 'CreateVIInput'.
    
    :var genericBOMLine: BOM Line of configured structure from which new Variant Item Structure should be created. This
    structure has Variants defined and can be configured for some given Variant Configuration. For more details please
    refer section 'Creating and updating variant items' from Teamcenter Documentation.
    :var newVariantItemRevision: Item Revision of newly created Variant Item.
    """
    genericBOMLine: BOMLine = None
    newVariantItemRevision: ItemRevision = None


@dataclass
class CreateVIResponse(TcBaseObj):
    """
    Response structure for operation ''createVariantItem''
    
    :var createVariantItemOutput: List of outputs one each for ''CreateVIInput''
    :var serviceData: Service Data to return error back to caller.
    """
    createVariantItemOutput: List[CreateVIOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DeepCopyData(TcBaseObj):
    """
    Structure to collect information about objects which should be copied or referenced in new variant Item.
    
    :var attachedObject: Other side object
    :var propertyName: Name of relation type or reference property for which DeepCopy rule is configured
    :var propertyType: Name of relation type or reference property for which DeepCopy rule is configured
    :var copyAction: DeepCopy action [NoCopy, CopyAsReference, CopyAsObject or Select]
    :var isTargetPrimary: Flag indicating if target object is primary or secondary
    :var isRequired: If this flag is false, the copy action can be dynamically changed by user
    :var copyRelations: This is a Boolean representing whether the Properties on the Relation if any in the Relation
    that exists between
    :var saveAsInputTypeName: SaveAsInput type name
    :var saveAsInput: SaveAsInput field to capture user inputs on the SaveAs dialog. NOTE: This field is unused in the
    getSaveAsDesc operation. It is used in the saveAsObjects operation
    :var childDeepCopyData: Vector of DeepCopy data for the secondary objects on the other side
    """
    attachedObject: BusinessObject = None
    propertyName: str = ''
    propertyType: PropertyType = None
    copyAction: str = ''
    isTargetPrimary: bool = False
    isRequired: bool = False
    copyRelations: bool = False
    saveAsInputTypeName: str = ''
    saveAsInput: SaveAsInput = None
    childDeepCopyData: List[DeepCopyData] = ()


class PropertyType(Enum):
    """
    Enumeration of the different property types for which DeepCopy Rule  configuration is enabled
    """
    Relation = 'Relation'
    Reference = 'Reference'


"""
DateMap
"""
DateMap = Dict[str, datetime]


"""
DateVectorMap
"""
DateVectorMap = Dict[str, List[datetime]]


"""
DoubleMap
"""
DoubleMap = Dict[str, float]


"""
DoubleVectorMap
"""
DoubleVectorMap = Dict[str, List[float]]


"""
FloatMap
"""
FloatMap = Dict[str, float]


"""
FloatVectorMap
"""
FloatVectorMap = Dict[str, List[float]]


"""
IntMap
"""
IntMap = Dict[str, int]


"""
IntVectorMap
"""
IntVectorMap = Dict[str, List[int]]


"""
AttributeNameValueMap
"""
AttributeNameValueMap = Dict[str, str]


"""
Map containing string property values
"""
StringMap = Dict[str, str]


"""
BoolMap
"""
BoolMap = Dict[str, bool]


"""
StringVectorMap
"""
StringVectorMap = Dict[str, List[str]]


"""
TagMap
"""
TagMap = Dict[str, BusinessObject]


"""
BoolVectorMap
"""
BoolVectorMap = Dict[str, List[bool]]


"""
TagVectorMap
"""
TagVectorMap = Dict[str, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.base import TcBaseObj
from tcsoa.gen.Internal.ImportExport._2010_04.L10NImportExport import TypePropertiesInfo
from dataclasses import dataclass


@dataclass
class ObjectsInfo2(TcBaseObj):
    """
    'ObjectsInfo2' structure contains parameter required to filter Teamcenter business objects based on selected
    criteria that can be used to export localized business objects for review or translation purpose.
    
    :var translationsExist: If true, mode is Review mode; otherwise, false, mode is Translate.
    Review mode allows user to export all localized business objects into XML file based on status and locale selected.
    Translate mode exports business objects into XML file for selected target locale.
    :var locale: The name of the desired locale. The valid locale name should be in the format as outlined in the Java
    Standard Language (i.e. en_US for English, United States). A full list of locales supported by the Teamcenter
    server can be obtained from the service operation
    'Teamcenter::Soa::Core::_2010_04::LanguageInformation.getLanguageList'. This can also be set to English word Master
    for exporting all business objects into XML file that needs to be translated to different locale. This parameter
    works in conjunction with "translationsExist" and "statuses".
    If "locale" is set to Master, it indicates translate mode and valid values for other parameters:
    "translationsExist" must be "false", In case of value is set to "true", the 'ServiceData' contains error 269002.
    "statuses" should be empty; this value is ignored for Master locale. If "locale" is set to any other valid locale
    value then possible values for other parameters and behavior is: "translationsExist" is set to "true" indicate
    review mode. This will export all the business objects for selected "statuses".
    :var statuses: A list of status to filter the localized values on. The status may be one or more of the following:
    "Approved"
    "In-Review"
    "Pending"
    "Invalid"
    This is ignored if mode is "translate" ( 'translationsExist' is false ) and selected locale is Master. In "review"
    mode nothing is exported in case of invalid "statuses" is passed and operation returns error 269002 in
    'ServiceData'.
    :var transferOptionSet: The name of the Transfer Option Set that is used while exporting objects. This is retrieved
    using 'Teamcenter::Soa::GlobalMultiSite::_2007_06::ImportExport::getAllTransferOptionSets' operation.
    :var objects: A list of Teamcenter business objects which need to be filtered or exported basedon the passed in
    parameters.
    :var typesPropertiesInfo: A list of type names and the corresponding properties.
    """
    translationsExist: bool = False
    locale: str = ''
    statuses: List[str] = ()
    transferOptionSet: str = ''
    objects: List[BusinessObject] = ()
    typesPropertiesInfo: List[TypePropertiesInfo] = ()



from __future__ import annotations

from tcsoa.gen.Internal.Query._2012_02.SavedQuery import DescribeSavedQueryDefinitionsResponse, DescribeSavedQueryDefinitionInput
from typing import List
from tcsoa.base import TcService


class SavedQueryService(TcService):

    @classmethod
    def describeSavedQueryDefinitions(cls, requestedQueries: List[DescribeSavedQueryDefinitionInput]) -> DescribeSavedQueryDefinitionsResponse:
        """
        Returns a complete list of all the clauses and sort attributes that make up a saved query.  This provides a
        complete list of information, including that which is internal from the customer.
        """
        return cls.execute_soa_method(
            method_name='describeSavedQueryDefinitions',
            library='Internal-Query',
            service_date='2012_02',
            service_name='SavedQuery',
            params={'requestedQueries': requestedQueries},
            response_cls=DescribeSavedQueryDefinitionsResponse,
        )



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2007_06.ScheduleManagement import ScheduleCopyOptionsContainer, TaskDeliverableContainer, DeleteNotificationRuleContainer, NotificationRulesList, GetNotificationRuleContainer, NotificationRuleContainer, MultiScheduleCopyResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def copySchedules(cls, scheduleCopyContainer: List[ScheduleCopyOptionsContainer]) -> MultiScheduleCopyResponse:
        """
        Makes a deep copy of the schedule with options to reset work and copy existing baselines.
        """
        return cls.execute_soa_method(
            method_name='copySchedules',
            library='ProjectManagement',
            service_date='2007_06',
            service_name='ScheduleManagement',
            params={'scheduleCopyContainer': scheduleCopyContainer},
            response_cls=MultiScheduleCopyResponse,
        )

    @classmethod
    def createOrUpdateNotificationRules(cls, notificationsContainers: List[NotificationRuleContainer]) -> ServiceData:
        """
        Creates a list of notification rules for Schedule or ScheduleTask based on the notifications container. You use
        notifications to notify individuals, including yourself, of important events associated with selected objects.
        Notifications utilize Teamcenter mail and the Subscription Manager. To receive notifications and subscriptions,
        a system administrator must set the value of the Mail_server_name preference to a name of a valid mail server
        (this task needs only to be performed once). The e-mail address in the Person object for every user that's
        expected to receive a notification.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateNotificationRules',
            library='ProjectManagement',
            service_date='2007_06',
            service_name='ScheduleManagement',
            params={'notificationsContainers': notificationsContainers},
            response_cls=ServiceData,
        )

    @classmethod
    def createTaskDeliverableTemplates(cls, taskDeliverableData: List[TaskDeliverableContainer]) -> ServiceData:
        """
        Creates new task deliverable template and relates them to the task. This is done by going through each
        deliverable, checks if the user has write access on the specified task, and then checking if the task
        deliverable already exists for the task.  If it does not exist it will create an instance of the task
        deliverable and add to the list of task deliverables.
        """
        return cls.execute_soa_method(
            method_name='createTaskDeliverableTemplates',
            library='ProjectManagement',
            service_date='2007_06',
            service_name='ScheduleManagement',
            params={'taskDeliverableData': taskDeliverableData},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteNotificationRules(cls, notificationRuleContainer: List[DeleteNotificationRuleContainer]) -> ServiceData:
        """
        Delete notification rules for Schedule or ScheduleTask based on the contents of the delete notifications
        container. After deleting the notification will rule, the users attached to that rule will not receive any more
        notifications for that specific action.
        """
        return cls.execute_soa_method(
            method_name='deleteNotificationRules',
            library='ProjectManagement',
            service_date='2007_06',
            service_name='ScheduleManagement',
            params={'notificationRuleContainer': notificationRuleContainer},
            response_cls=ServiceData,
        )

    @classmethod
    def getNotificationRules(cls, notificationRuleContainer: List[GetNotificationRuleContainer]) -> NotificationRulesList:
        """
        Get a list of notification rules for Schedule or ScheduleTask based on the notifications container. Use
        notifications to notify individuals, including yourself, of important events associated with selected objects.
        Notifications utilize Teamcenter mail and the Subscription Manager. To receive notifications and subscriptions,
        a system administrator must set the value of the Mail_server_name preference to a name of a valid mail server
        (this task needs only to be performed once). The e-mail address in the Person object for every user that's
        expected to receive a notification.
        """
        return cls.execute_soa_method(
            method_name='getNotificationRules',
            library='ProjectManagement',
            service_date='2007_06',
            service_name='ScheduleManagement',
            params={'notificationRuleContainer': notificationRuleContainer},
            response_cls=NotificationRulesList,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Rdv._2009_04.VariantManagement import DeleteNamedVariantExpressions, GetSVRResponse, GetSVRInfo, GetMultipleNVEResponse, MultipleNamedVariantExpressions
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class VariantManagementService(TcService):

    @classmethod
    def createMultipleNamedVariantExpressions(cls, input: MultipleNamedVariantExpressions) -> GetMultipleNVEResponse:
        """
        Creates a list of Named Variant Expressions (NVEs) based on the input structure. When Architecture Breakdown is
        using shared NVEs, the NVE gets created on the Architecture Breakdown. Architecture Breakdown is the topline in
        Architecture Breakdown structure. The payload variant expression passed as part of the input structure is used
        to create the NVE. The code is used as the name of the NVE. When using the shared NVEs, if the Architecture
        object or variant expression is not supplied then this operation will fail. It is required that applicable
        variability is assigned to the toplevel Architecture Breakdown before creating the NVE. When shared NVEs is
        used, all NVEs with in Architecture Breakdown must have a unique name. Each NVE should have a unique variant
        condition. When not using the shared NVEs, NVEs get created on the selected Architecture Breakdown Element.
        
        Use cases:
        The NVEs are created in the Architecture Breakdown structure as raw material for constructing the final variant
        condition on the design assembly. NVEs are applied on the design assembly during Replace Design in Product
        process.
        """
        return cls.execute_soa_method(
            method_name='createMultipleNamedVariantExpressions',
            library='Internal-Rdv',
            service_date='2009_04',
            service_name='VariantManagement',
            params={'input': input},
            response_cls=GetMultipleNVEResponse,
        )

    @classmethod
    def createSavedVariantRules(cls, input: GetSVRInfo) -> GetSVRResponse:
        """
        This operation creates multiple Saved Variant Rules (SVR) in order to configure a particular variant of a
        structure. The variant rule presents all the options that are used in the structure for which user can then set
        values as required. User does not have to specify values for all option, they can remain unset.
        SVRs created for configuring the part solutions are saved under the top level architecture item. If SVR already
        exists for the provided ItemRevision object, the operation will modify existing SVR with the default variant
        rule on which the option values are set.
        The values needed for creating SVR(s) is provided through the 'GetSVRInfo' object which is passed as a
        parameter input to this operation.
        
        Use cases:
        Use Case 1: Creating Saved Variant Rule(s)
        This operation is used when a user needs to create/modify one or more Saved Variant Rules in one go. All the
        details need to be specified in a list of 'GetSVRInfo' objects. The operation first attempts to find the
        existence of supplied SVR name, if it exists then SVR is modified otherwise new one is created. It is to be
        noted that only those 'SVRData' objects are processed whose both attributes option and value are set, failing
        to supply either of them will ignore the SVR processing.
        """
        return cls.execute_soa_method(
            method_name='createSavedVariantRules',
            library='Internal-Rdv',
            service_date='2009_04',
            service_name='VariantManagement',
            params={'input': input},
            response_cls=GetSVRResponse,
        )

    @classmethod
    def deleteMultipleNamedVariantExpressions(cls, input: DeleteNamedVariantExpressions) -> ServiceData:
        """
        Deletes the Named Variant Expressions (NVEs). When NVEs get deleted its reference from Architecture breakdown
        also gets deleted. NVE is owned by the Architecture Breakdown object. Input object contains the information of
        the NVEs that need to be deleted. If Architecture Breakdown object is not supplied then this operation will
        fail.
        
        Use cases:
        This operation can be used to delete the unused or wrongly created NamedVariantExpressions by passing the
        Architecture Breakdown object which owns the Named Variant Expression and list of NamedVariantExpression
        objects as a part of DeleteNamedVariantExpressions object.
        """
        return cls.execute_soa_method(
            method_name='deleteMultipleNamedVariantExpressions',
            library='Internal-Rdv',
            service_date='2009_04',
            service_name='VariantManagement',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, TCCalendar, ScheduleTask, Schedule, ResourceAssignment, TaskDependency, TCCalendarEvent
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetNotificationRuleContainer(TcBaseObj):
    """
    The input information needed to find all notifications for a particular task or schedule.
    
    :var target: The tag of a Schedule or ScheduleTask.
    :var subscriber: The tag of the subscriber.
    """
    target: BusinessObject = None
    subscriber: BusinessObject = None


@dataclass
class MultiScheduleCopyResponse(TcBaseObj):
    """
    Container to hold multiple ScheduleCopyResponse objects.
    
    :var scheduleResponse: A collection of the schedule responses.
    :var serviceData: The ServiceData.
    """
    scheduleResponse: List[ScheduleCopyResponse] = ()
    serviceData: ServiceData = None


@dataclass
class NotificationRuleContainer(TcBaseObj):
    """
    The information needed for the creation of a single Notification rule.
    
    :var target: The tag of a schedule or task
    :var subscriber: The subscriber for this notification.  It is the Target object in the case of a notification or a
    User object in the case of a subscription.
    :var notificationSubject: Subject of the email. Valid value- can be empty string
    :var notificationMessage: Message of the e-mail. Valid value- can be empty string
    :var recipient: The Users, groups, disciplines, or schedule members to be notified.
    :var additionalEmails: A semi-colon separated list of e-mail addresses. Valid value - can be empty string
    :var notificationCondition: The condition data for notifications that are triggered conditionally.  E.g.  Schedule
    Task near due. Valid value - can be empty string
    :var ruleType: The type of rule to create. Valid values are
    {__Add_Task,__Delete_Task,__Near_Due,__Overdue,__Start_Date_Change,__Finish_Date_Change,__Status_Change,__Status_ChangeTo,__Priority_Change,__Priority_ChangeTo,__Work_Estimate_Change,__Work_Complete_Change,__Work_Ready,__User_Assigned
    }
    :var status: Active or inactive status of the notification.
    :var update: Attempting to update an existing rule.
    """
    target: BusinessObject = None
    subscriber: BusinessObject = None
    notificationSubject: str = ''
    notificationMessage: str = ''
    recipient: List[BusinessObject] = ()
    additionalEmails: str = ''
    notificationCondition: str = ''
    ruleType: str = ''
    status: bool = False
    update: bool = False


@dataclass
class NotificationRulesList(TcBaseObj):
    """
    A collection of the single NotificationRuleContainer objects.
    
    :var notificationRules: The collection of Notification Rules.
    :var serviceData: The ServiceData.
    """
    notificationRules: List[NotificationRuleContainer] = ()
    serviceData: ServiceData = None


@dataclass
class ScheduleCopyOptionsContainer(TcBaseObj):
    """
    The input information necessary to copy a schedule.
    
    :var name: The name of the new schedule.
    :var description: The description of the new schedule.
    :var scheduleToCopy: The tag to the schedule to copy.
    :var resetWork: Flag to indicate whether or not to reset the tasks' execution data (%, status, work complete, etc).
    :var copyBaselines: Flag to indicate whether or not to copy baselines.
    :var loadOnResponse: Flag to indicate whether or not to load the schedule in the response.
    """
    name: str = ''
    description: str = ''
    scheduleToCopy: Schedule = None
    resetWork: bool = False
    copyBaselines: bool = False
    loadOnResponse: bool = False


@dataclass
class ScheduleCopyResponse(TcBaseObj):
    """
    The response from a schedule copy call.  It includes all the data necessary to load that schedule.
    
    :var schedule: The schedule.
    :var scheduleTask: The schedule tasks.
    :var resourceAssignment: The task assignments.
    :var taskDependency: The task dependencies.
    :var calendar: The calendars asssociated with the schedule and its members.
    :var calendarEvent: The calendar events associated with the calendars.
    """
    schedule: Schedule = None
    scheduleTask: List[ScheduleTask] = ()
    resourceAssignment: List[ResourceAssignment] = ()
    taskDependency: List[TaskDependency] = ()
    calendar: List[TCCalendar] = ()
    calendarEvent: List[TCCalendarEvent] = ()


@dataclass
class TaskDeliverableContainer(TcBaseObj):
    """
    The input information for a single task deliverable.
    
    :var scheduleTask: The task which will contain this delievable. Tag of the task
    :var scheduleDeliverable: The ScheduleDeliverable to reference. Tag of schedule deliverable.
    :var submitType: The submit type  (3=Don't submit, 0=submit as target, 1=submit as reference).
    """
    scheduleTask: ScheduleTask = None
    scheduleDeliverable: BusinessObject = None
    submitType: int = 0


@dataclass
class DeleteNotificationRuleContainer(TcBaseObj):
    """
    The input information needed to delete an existing notification rule.
    
    :var target: The tag  of  the Schedule  or ScheduleTask.
    :var subscriber: The tag of the subscriber.
    :var ruleType: The type of rule to delete. Valid values-{
    __Add_Task,__Delete_Task,__Near_Due,__Overdue,__Start_Date_Change,__Finish_Date_Change,__Status_Change,__Status_ChangeTo,__Priority_Change,__Priority_ChangeTo,__Work_Estimate_Change,__Work_Complete_Change,__Work_Ready,__User_Assigned}
    """
    target: BusinessObject = None
    subscriber: BusinessObject = None
    ruleType: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExtendedProperties(TcBaseObj):
    """
    Extended metadata properties for an attribute.
    
    :var propertyName: An extended property name.
    :var propertyValue: The extended property values.
    """
    propertyName: str = ''
    propertyValue: List[str] = ()


@dataclass
class GetLogicalObjectInput(TcBaseObj):
    """
    Holds the list of the root object instances and the logical type name to retrieve the matched logical object
    instances.
    
    :var rootObjects: A list of root object instances. Persistent types are the only supported business objects for a
    root object.
    :var loTypeName: Name of the logical object type for which the logical object instances are to be searched for. The
    logical object type name corresponds to the name of any subtype of Fnd0LogicalObject.
    """
    rootObjects: List[BusinessObject] = ()
    loTypeName: str = ''


@dataclass
class GetLogicalObjectOutput(TcBaseObj):
    """
    Holds the list of the retrieved logical object instances.
    
    :var logicalObjects: A list of the retrieved logical object instances.
    """
    logicalObjects: List[LogicalObjectOutput] = ()


@dataclass
class GetLogicalObjectResponse(TcBaseObj):
    """
    Holds the list of the retrieved logical object output response and partial errors.
    
    :var serviceData: Returned service data.
    :var loOutputs: A list of the retrieved logical object output response corresponding to a  logical object input.
    :var classificationAttributeDesc: A map of classification objects and a list of references to their attribute
    descriptors.
    :var classificationObjectInfo: A map of the ICO ids and their class attribute info.
    """
    serviceData: ServiceData = None
    loOutputs: List[GetLogicalObjectOutput] = ()
    classificationAttributeDesc: ClassificationAttributeDescMap = None
    classificationObjectInfo: ClassificationObjectInfoMap = None


@dataclass
class AttrDescriptor(TcBaseObj):
    """
    Structure containing map of class attribute descriptors.
    
    :var attributeDescMap: The map of the attribute descriptors.
    """
    attributeDescMap: AttributeDescriptorMap = None


@dataclass
class LogicalObjectOutput(TcBaseObj):
    """
    This structure contains the logical object instance and its related classification objects information.
    
    :var root: An instance of the root object.
    :var logicalObjectInstances: A list of logical object instances. Only one logical object instance is returned for a
    given root object in the current implementation.
    :var memberClassificationObjects: A list of  MemberClassificationObject objects.
    """
    root: BusinessObject = None
    logicalObjectInstances: List[BusinessObject] = ()
    memberClassificationObjects: List[MemberClassificationObject] = ()


@dataclass
class MemberClassificationObject(TcBaseObj):
    """
    Contains classification object information.
    
    :var memberOrRootName: The internal name of the member or the root.
    :var icoIDs: A list of ICO ids.
    """
    memberOrRootName: str = ''
    icoIDs: List[str] = ()


@dataclass
class AttributeValue(TcBaseObj):
    """
    Structure for each ICO attribute value.
    
    :var attrValue: The value of the attribute.
    :var unit: The unit of the attribute.
    :var locale: The locale for this attribute.
    """
    attrValue: str = ''
    unit: str = ''
    locale: str = ''


@dataclass
class AttributeValues(TcBaseObj):
    """
    Contains values for ICO attributes and a boolean to check whether attribute is valid.
    
    :var isAttributeValid: A flag to indicate whether the attribute is valid.
    :var values: A list of values for this attribute.
    """
    isAttributeValid: bool = False
    values: List[AttributeValue] = ()


@dataclass
class ClassAttrDescriptor(TcBaseObj):
    """
    Structure representing Class attribute description.
    
    :var id: An attribute id.
    :var name: An attribute name.
    :var metricMinValue: An attribute metric min value.
    :var metricMaxValue: An attribute metric max value.
    :var nonMetricFormat: An attribute non metric format.
    :var nonMetricUnit: An attribute non metric format.
    :var nonMetricDefaultValue: An attribute non metric default value.
    :var nonMetricMinValue: An attribute non metric min value.
    :var nonMetricMaxValue: An attribute non metric max value.
    :var extendedProps: The extended properties.
    :var shortName: An attribute short name.
    :var descripton: An attribute description.
    :var annotation: An attribute annotation.
    :var arraySize: An attribute array size.
    :var options: An attribute options.
    :var metricFormat: An attribute metric format.
    :var metricUnit: An attribute metric unit.
    :var metricDefaultValue: An attribute metric default value.
    """
    id: str = ''
    name: str = ''
    metricMinValue: str = ''
    metricMaxValue: str = ''
    nonMetricFormat: int = 0
    nonMetricUnit: str = ''
    nonMetricDefaultValue: str = ''
    nonMetricMinValue: str = ''
    nonMetricMaxValue: str = ''
    extendedProps: List[ExtendedProperties] = ()
    shortName: str = ''
    descripton: str = ''
    annotation: str = ''
    arraySize: int = 0
    options: int = 0
    metricFormat: int = 0
    metricUnit: str = ''
    metricDefaultValue: str = ''


@dataclass
class ClassAttributeInfo(TcBaseObj):
    """
    Structure containing classification object information.
    
    :var classId: The classification class Id.
    :var attributeValuesMap: The class attributes map.
    :var unitBase: A class unit system of measure.
    :var classifiedObject: Reference of the classified WorkspaceObject object. This can be a NULLTAG if it is a
    standalone ICO.
    """
    classId: str = ''
    attributeValuesMap: AttributeValuesMap = None
    unitBase: ClassUnitBase = None
    classifiedObject: BusinessObject = None


class ClassUnitBase(Enum):
    """
    Enumerates the list of value for class unit system.
    """
    CLS_METRIC = 'CLS_METRIC'
    CLS_NONMETRIC = 'CLS_NONMETRIC'
    CLS_BOTH = 'CLS_BOTH'


"""
Contains descriptor of an attribute.
"""
AttributeDescriptorMap = Dict[str, ClassAttrDescriptor]


"""
Contains attribute Id and corresponding value(s).

&bull;primary key :  The attribute ID. 
&bull;values :            A value structure for this attribute.
"""
AttributeValuesMap = Dict[str, AttributeValues]


"""
Contains class Id and corresponding attribute descriptor.
"""
ClassificationAttributeDescMap = Dict[str, AttrDescriptor]


"""
Contains classification object information.

&bull;primary key:  The classification object (ICO) ID. 
&bull;values:            A reference of the structure containing additional ICO properties.
"""
ClassificationObjectInfoMap = Dict[str, ClassAttributeInfo]



from __future__ import annotations

from tcsoa.gen.StructureManagement._2012_02.StructureVerification import ACInput
from tcsoa.gen.BusinessObjects import Dataset
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ACFavoriteInfo(TcBaseObj):
    """
    Accountability check settings which can be used to run accountability check or to load the settings in the
    accountability check dialog. The following partial errors may be returned in ServiceData element.
        204045        Input dataset UID is not a valid dataset of accountability check favorite.
        204047        The accountability check favorites XML is invalid. It may contain the entries that               
         cannot be evaluated by XML parser.
    
    
    :var accSettings: Accountability check settings.
    :var serviceData: Teamcenter service data.
    """
    accSettings: ACInput = None
    serviceData: ServiceData = None


@dataclass
class ACFavoritesInput(TcBaseObj):
    """
    The required parameters for the manageACFavorites method.
    
    :var accSettings: Accountability check settings to save. May have empty values for update and delete actions.
    :var name: Name of the dataset containing accountability check settings of the favorite.  May have empty string for
    update and delete actions.
    :var description: Description of the favorite that is being created or updated. May have empty string for update
    and delete actions.
    :var datasetUID: UID of the dataset to be updated or deleted . If the value is empty a new dataset is created. 
    :var action: Specifies the action to be performed on the dataset. Possible values are "create", "update" or
    "delete".
    """
    accSettings: ACInput = None
    name: str = ''
    description: str = ''
    datasetUID: str = ''
    action: str = ''


@dataclass
class ACFavoritesResponse(TcBaseObj):
    """
    The created or updated DataSet. The following partial errors may be returned:
        204042        Input dataset uid is not a valid dataset to update the accountability check favorites.
        204043        No transient volume directory, cannot create favorites xml file.
        204044        Invalid accountability check favorites xml.
        204045        Dataset missing named reference file.
        204046        Parser unable to parse xml named reference file.
        204047        Dataset is not text type.
    
    
    :var dataset: Dataset containing the accountability check settings.
    :var serviceData: Teamcenter service data.
    """
    dataset: Dataset = None
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.base import TcService


class UserManagementService(TcService):

    @classmethod
    def resetUserPassword(cls, userid: str, oldpassword: str, newpassword: str) -> bool:
        """
        Changes password of a user. A login user is allowed to modify his(her) own password. As of Teamcenter 11.3, a
        system administrator or privileged user is allowed to modify other users's passwords. This operation will throw
        a ServiceException as described below.".
        
        Use cases:
        This operation is used by the Active Workspace user interface.
        
        Exceptions:
        >Service Exception
        
        10729: Either the specified user ID is not the current session user, or the logged-in user is not a system
        administrator or an authorized user.
        """
        return cls.execute_soa_method(
            method_name='resetUserPassword',
            library='Internal-Administration',
            service_date='2015_10',
            service_name='UserManagement',
            params={'userid': userid, 'oldpassword': oldpassword, 'newpassword': newpassword},
            response_cls=bool,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import User, Group, Discipline, Role
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateDisciplinesIn(TcBaseObj):
    """
    This structure holds a DisciplineObject object which specifies the values of a new Discipline object. as well as
    the parent group for the new discipline.
    
    :var discipline: The object with initial data for the creation of a new Discipline object.
    :var group: The parent group of the new Discipline object.
    :var role: The role for the Discipline object. However it is not currently supported by this operation.
    """
    discipline: DisciplineObject = None
    group: Group = None
    role: Role = None


@dataclass
class CreateDisciplinesOutput(TcBaseObj):
    """
    This structure holds the newly created Discipline object and corresponding client ID.
    
    :var clientId: Identifying string from the source DisciplineObject.
    :var discipline: The new Discipline object created in this operation.
    """
    clientId: str = ''
    discipline: Discipline = None


@dataclass
class CreateDisciplinesResponse(TcBaseObj):
    """
    This structure holds a list of newly created Discipline objects.
    
    :var output: List of CreateDisciplinesOutput objects, one for each CreateDisciplinesIn object.
    :var serviceData: The object which holds the partial errors that occurred during creation of new Discipline objects
    with all newly created discipline object in its created object list.
    """
    output: List[CreateDisciplinesOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DisciplineLevel(TcBaseObj):
    """
    Struct containing the discipline level name and number attribute information.
    
    :var levelName: levelName
    :var levelNumber: levelNumber
    """
    levelName: str = ''
    levelNumber: int = 0


@dataclass
class DisciplineObject(TcBaseObj):
    """
    This structure holds initial property values for a new discipline object and the corresponding client ID.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify returned CreateDisciplinesOutput
    element and partial errors associated with the DisciplineObject input.
    :var name: The name of the new Discipline object to be created.
    :var description: The description text of the new Discipline object to be created.
    :var defaultRate: The default rate property of the new Discipline object to be created.
    :var levels: The list of Discipline level.
    :var users: The list of DisciplineUser objects which specifies user members for the new Discipline to be created.
    """
    clientId: str = ''
    name: str = ''
    description: str = ''
    defaultRate: float = 0.0
    levels: List[DisciplineLevel] = ()
    users: List[DisciplineUser] = ()


@dataclass
class DisciplineUser(TcBaseObj):
    """
    This structure  holds a user and its discipline level name.
    
    :var user: The Teamcenter User object to be added to the new discipline.
    :var levelName: Not supported.
    """
    user: User = None
    levelName: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AattachmentsCreated(TcBaseObj):
    """
    Details about created attachment.
    
    :var sourceLine: The BOMLine for which attachment line is created. This is same as the incoming source line.
    :var createdObject: The newly created dataset or the form.
    :var attachmentLine: The newly created attachment line.
    :var writeFileTicket: The FMS write-file-ticket of the file.
    """
    sourceLine: BusinessObject = None
    createdObject: BusinessObject = None
    attachmentLine: BusinessObject = None
    writeFileTicket: str = ''


@dataclass
class AttachmentLines(TcBaseObj):
    """
    Information about the attachments of a BOMLine.
    
    :var serviceData: The service data containing partial errors. The following partial error may be returned.
    - 253148: No attachment lines found for the  line.
    
    
    :var attachmentsInfo: The map of the BOMLine and a structure holding the information about its attachments.
    """
    serviceData: ServiceData = None
    attachmentsInfo: AttachmentInfoMap = None


@dataclass
class AttachmentResponse(TcBaseObj):
    """
    The response.
    
    :var attchments: The list of the structure "attachmentsCreated" specifying the details about created attachment.
    :var serviceData: Service data containing partial errors.
    """
    attchments: List[AattachmentsCreated] = ()
    serviceData: ServiceData = None


@dataclass
class AttachmentWindowInfo(TcBaseObj):
    """
    The information of the attachment window.
    
    :var attachmentWindow: Attachment window of type CfgAttachmentWindow.
    :var attachmentLines: The list of CfgAttachmentLine objects representing the attachment lines in the attachment
    window. This first member of the list will always be the top line of the attachment window.
    """
    attachmentWindow: BusinessObject = None
    attachmentLines: List[BusinessObject] = ()


@dataclass
class AttachmentsInput(TcBaseObj):
    """
    The input contains a BOMLine object for which attachment lines are required.
    The input contains the list of CfgAttachmentWindow objects too.
    If the attachment window list is not empty, the given line is set as top line of all the attachment windows in the
    list and its attachments are fetched.
    
    If the list is empty, a new attachment window is created for the incoming line.
    
    :var line: The BOMLine objects for which attachment lines are required.
    :var attachmentWindowsList: The list of attachment window of type CfgAttachmentWindow.
    This list may be empty and in that case a new attachment window is created for the given BOMLine. If the list is
    not empty, the given BOMLine is set as the top line of all the attachment windows in the list.
    """
    line: BusinessObject = None
    attachmentWindowsList: List[BusinessObject] = ()


@dataclass
class CreateAttachmentsInput(TcBaseObj):
    """
    The structure related to attachment line information.
    
    :var sourceLine: The BOMLine or its subtype for which attachment is to be created. Example of subtypes are
    Mfg0BvrProcess, Mfg0BvrOperation, MfgoBvrPlantBOP etc.
    :var targetObject: The attachment line object under which the new attachment is to be created, e.g.,  top
    attachment line, pseudo folder etc.
    :var importFileType: Type of import file. It can be either Text or Binary.
    Ignored if importFilePath is blank.
    Ignored if the objectType is specified as Form.
    :var namedRefType: Type of named reference of the dataset. For example JT, Text ext.
    Ignored if the objectType is specified as Form.
    :var namedRefSubType: Sub type of named ref. Can be blank.
    Ignored if the objectType is specified as Form.
    :var relation: The name of the relation between dataset/form and ItemRevision.
    A default relation will be used if this is blank.
    :var saveForm: To save form or not. Ignored if the objectType is specified as Dataset.
    :var dsContainer: The object to which dataset is attached. If Null then Dataset is attached to the ItemRevision. 
    Ignored if the objectType is specified as Form.
    :var objectType: Specifies the type of object to be created for the attachment line. It can have two values
    - Dataset
    - Form
    
    
    :var name: Name.
    :var description: Description
    :var type: Specific type of the dataset or the form. For example Direct Model, Text, ArcWeld Attribut Form Time Way
    Plan Location Form etc. This cannot be blank.
    :var datasetID: Dataset ID. This can be blank. In that case the dataset with default id is created.
    Ignored if the objectType is specified as Form.
    :var datasetRev: Dataset revision. This can be blank. In that case the dataset with default revision is created.
    Ignored if the objectType is specified as Form.
    :var toolUsed: Name of tool to be used to open the dataset. For example Text Editor.
    Ignored if the objectType is specified as Form.
    :var importFilePath: Full path of the file to be imported with the dataset. Can be blank.
    Ignored if the objectType is specified as Form.
    """
    sourceLine: BusinessObject = None
    targetObject: BusinessObject = None
    importFileType: str = ''
    namedRefType: str = ''
    namedRefSubType: str = ''
    relation: str = ''
    saveForm: bool = False
    dsContainer: BusinessObject = None
    objectType: str = ''
    name: str = ''
    description: str = ''
    type: str = ''
    datasetID: str = ''
    datasetRev: str = ''
    toolUsed: str = ''
    importFilePath: str = ''


"""
The map of the BOMLine and a structure holding the information about its attachments.
"""
AttachmentInfoMap = Dict[BusinessObject, AttachmentWindowInfo]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExtensionAttachmentTypeDetails(TcBaseObj):
    """
    ExtensionAttachmentTypeDetails represents a list of AttachmentTypeDetailElement for each extension name.
    
    :var extensionName: Extension name that needs to be compared.
    An extension name is the ID of a "serverExtension" registered in Accountability Check&rsquo;s plugin.xml. It can be
    located in com.teamcenter.rac.cme.accountabilitycheck.relations package. The server extension for Attachments is
    "Mfg0dataset_form".
    :var attachmentTypesDetails: A list of details of each attachment type.
    """
    extensionName: str = ''
    attachmentTypesDetails: List[AttachmentTypeDetailElement] = ()


@dataclass
class AttachmentComparisonDetailsResponse(TcBaseObj):
    """
    AttachmentComparisonDetailsResponse represents a list of AttachmentTypeDetails element and ServiceData.
    
    :var details: A list of AttachmentType detail elements - one for each input equivalent set.
    :var serviceData: Object that captures any partial errors.
    """
    details: List[AttachmentTypeDetail] = ()
    serviceData: ServiceData = None


@dataclass
class AttachmentDetail(TcBaseObj):
    """
    AttachmentDetail holds a set of attachments&rsquo; tag and match type information for each line in input equivalent
    set.
    
    :var attachments: For each BOMLine in input equivalent set holds the attachment objects. Attachments can be both
    Dataset and Form objects.
    :var matchType: To indicate the type of match between attachments.
    Probable values can only be from the set {0, 1, 2} which represents the set {"Full Match", "Source Modified",
    "Target Modified"} respectively.
    """
    attachments: List[BusinessObject] = ()
    matchType: int = 0


@dataclass
class AttachmentTypeDetail(TcBaseObj):
    """
    AttachmentTypeDetail pairs the ExtensionAttachmentTypeDetails element and a list of equivalent BOMLines with an
    index.
    
    :var index: Index of the input equivalent set for which these details were calculated.
    :var equivalentLines: A list of all equivalent lines in the input equivalent set (all equivalent sources in
    sequence and then all targets in sequence) of MEProcess,  MEOp and MEWorkarea for which attachments are to be
    compared.
    :var attachmentExtDetails: A list of attachment type details of this equivalent set.
    """
    index: int = 0
    equivalentLines: List[BusinessObject] = ()
    attachmentExtDetails: List[ExtensionAttachmentTypeDetails] = ()


@dataclass
class AttachmentTypeDetailElement(TcBaseObj):
    """
    AttachmentTypeDetailElement contains a list of all the attachments for a particular attachment relation type.
    
    :var attachmentRelType: The relation type of attachment (Cps0LBRel, IMAN_SPECIFICATION, Cps0RobotProgRel etc.)
    :var attachmentsDetails: A list of details of the attachment for each line in input equivalent set.
    """
    attachmentRelType: BusinessObject = None
    attachmentsDetails: List[AttachmentDetail] = ()



from __future__ import annotations

from tcsoa.gen.Internal.ConfigFilterCriteria._2013_12.VariantManagement import ConfiguratorServiceResponse
from tcsoa.base import TcService


class VariantManagementService(TcService):

    @classmethod
    def executeConfiguratorService(cls, inputXMLString: str, configuratorURL: str) -> ConfiguratorServiceResponse:
        """
        This service implements a mechanism to process the following configurator services:
        CheckOverlap
        ConvertExpressionToString
        ConvertStringToExpression
        GetPersistentID
        GetProductVariability
        GetTrueAvailability
        GetValidFeatureCombinations
        ValidateFeatureExpression
        ValidateAndCompleteFeatureCombination
        Consumers of this service need to provide an XML string as input and will receive an XML string as output. The
        input and output XML strings will be as per the XML schema definitions located in DMS at
        "//plm/tcpmm10.1.1/tcbom/configurator/xids".
        These XML schema definitions follow standard Teamcenter versioning procedure.
        
        Exceptions:
        >This operation may raise a Teamcenter::Soa::Server::ServiceException wrapping around the following Teamcenter
        errors:
        
        92001    An internal error has occurred in the "Fnd0SoaConfigFilterCriteria" module.
        92002    The Product Name and Product Namespace must not be empty.
        256000    An internal configurator error occurred.
        256001    A general configurator error occurred.
        256002    The following product is invalid: [DictionaryNamespace]FamilyName (Context: info).
        256003    The following feature value is invalid: [DictionaryNamespace]FamilyName (Context: info).
        256004    The following feature family is invalid: [DictionaryNamespace]FamilyName (Context: info).
        256005    An invalid feature combination was found (Context: info).
        256006    An invalid product configuration was found (Context: info).
        256007    An invalid expression was found (Context: info).
        256008    The following Boolean operator is invalid: opcode (Context: info).
        256009    An invalid effectivity date was found (Context: info).
        256010    A time-out occurred (Context: info).
        256011    A resource is not available (Context: info).
        256012    The following functionality is not implemented: function (Version: TcVersion).
        256013    A configurator implementation specific error occurred (Context: info).
        256014    The variant expression contains unmatched quote at position number.
        256015    The variant expression contains unmatched bracket at position number.
        256016    An unexpected token 'text' was found at position number.
        256017    The entry 'text' was found at position number when an option value was expected.
        256018    The token 'text' is missing after 'text' at position number.
        256019    The variant expression adaptor does not support text used at position number.
        256020    The XML tag 'text' has no text value.
        256021    The string 'formula' cannot be converted into an expression. 
        256022    The operator code 'opcode' in token at position number is unknown.
        256023    The variant value 'text' in token at position number is unknown.
        256024    The variant family 'text' in token at position number is unknown.
        256025    An ambiguous option value name 'FamilyName' in namespace 'DictionaryNamespace' in token at position
        number was found: text.
        256026    An ambiguous option family name 'FamilyName' in token at position number was found: 'text'.
        256027    The option namespace 'DictionaryNamespace' in token at position number is invalid.
        256028    The variant family 'FamilyName' in token at position number is invalid.
        256029    The variant option value 'text' for family 'FamilyName' in token at position number is invalid.
        256030    The variant option value 'text' for family 'FamilyName' in token at position number is invalid: its
        value is not numeric.
        256031    The variant option value 'text' for family 'FamilyName' in token at position number is invalid: the
        value unit of measure 'text' does not match the family unit of measure 'text'.
        256032    The option / family / family namespace combination 'text' / 'FamilyName' / 'DictionaryNamespace' in
        token at position number is invalid.
        256033    The family 'FamilyName' for option / family namespace combination 'text' / 'DictionaryNamespace' in
        token at position number is invalid. 
        256034    There is no option value in token at position number.
        256035    The variant option value 'text' for family 'FamilyName' in token at position 'number' is invalid: the
        value is not a valid date.
        """
        return cls.execute_soa_method(
            method_name='executeConfiguratorService',
            library='Internal-ConfigFilterCriteria',
            service_date='2013_12',
            service_name='VariantManagement',
            params={'inputXMLString': inputXMLString, 'configuratorURL': configuratorURL},
            response_cls=ConfiguratorServiceResponse,
        )



from tcsoa.gen.Vendormanagement._2012_09.services import VendorManagementService as imp0
from tcsoa.gen.Vendormanagement._2008_06.services import VendorManagementService as imp1
from tcsoa.gen.Vendormanagement._2007_06.services import VendorManagementService as imp2
from tcsoa.gen.Vendormanagement._2012_02.services import VendorManagementService as imp3
from tcsoa.gen.Vendormanagement._2016_09.services import VendorManagementService as imp4
from tcsoa.base import TcService


class VendorManagementService(TcService):
    addRemoveVendorRoles = imp0.addRemoveVendorRoles
    changeVendor = imp1.changeVendor
    createOrUpdateBidPackages = imp2.createOrUpdateBidPackages
    createOrUpdateLineItems = imp2.createOrUpdateLineItems
    createOrUpdateLineItemsWithType = imp3.createOrUpdateLineItemsWithType
    createOrUpdateVendorParts = imp2.createOrUpdateVendorParts
    createOrUpdateVendors = imp2.createOrUpdateVendors
    createVendorParts = imp4.createVendorParts
    deleteLineItems = imp2.deleteLineItems
    deleteVendorRoles = imp2.deleteVendorRoles
    deleteVendors = imp2.deleteVendors
    getVendorPartsWithSelRule = imp1.getVendorPartsWithSelRule



from __future__ import annotations

from tcsoa.gen.DocumentManagement._2008_06.DocumentControl import GetCheckinModeAndFilesOutputsResponse, GetAdditionalFilesForCheckinInputs, PostCreateResponse, PostCreateInputs, GetAdditionalFilesForCheckinOutputsResponse
from tcsoa.gen.BusinessObjects import ItemRevision
from typing import List
from tcsoa.base import TcService


class DocumentControlService(TcService):

    @classmethod
    def postCreate(cls, input: List[PostCreateInputs]) -> PostCreateResponse:
        """
        This operation is to be called by the client after the creation of the ItemRevision business object, if there
        are local files to be attached to the newly created ItemRevision. For ItemRevision under Item Revision
        Definition Control (IRDC) it will replace any datasets copied from a template with new datasets for the local
        files. The client will then need to import the local files in the volume based on the return information from
        the SOA.  For ItemRevision not under IRDC control, the commitInfos list field from the return 'PostCreateInfo'
        structure for this ItemRevision will be empty.
        
        Use cases:
        Create new item from the RAC
        During the new item create on RAC, if the ItemRevision business object is under IRDC control,  the Attach Files
        panel will be enabled in the create wizard dialog,  if user choose to attach local files in the Attach Files
        panel, the template files for the IRDC will be replaced for the newly created ItemRevision business object, and
        instead the new datasets will be created for the local files; if user choose not to attach any local files in
        the Attach Files panel, then the template files for the IRDC will be used for the newly created ItemRevision.
        """
        return cls.execute_soa_method(
            method_name='postCreate',
            library='DocumentManagement',
            service_date='2008_06',
            service_name='DocumentControl',
            params={'input': input},
            response_cls=PostCreateResponse,
        )

    @classmethod
    def getAdditionalFilesForCheckin(cls, inputs: List[GetAdditionalFilesForCheckinInputs]) -> GetAdditionalFilesForCheckinOutputsResponse:
        """
        This operation is used in conjunction with the getCheckinModeAndFiles operation during the Check In process.
        getCheckinModeAndFiles takes a list of ItemRevision objects that have been checked out, and returns the list of
        source files that have been downloaded to the client. This operation takes the list of downloaded files
        returned by getCheckinModeAndFiles, and returns the subset of those files that are eligible for Check In.
        
        Use cases:
        Check In
        
        This method is called after the getCheckinModeAndFiles operation, but before the files are checked in.
        """
        return cls.execute_soa_method(
            method_name='getAdditionalFilesForCheckin',
            library='DocumentManagement',
            service_date='2008_06',
            service_name='DocumentControl',
            params={'inputs': inputs},
            response_cls=GetAdditionalFilesForCheckinOutputsResponse,
        )

    @classmethod
    def getCheckinModeAndFiles(cls, inputs: List[ItemRevision]) -> GetCheckinModeAndFilesOutputsResponse:
        """
        Get the CheckIn mode and files for ItemRevision business objects.  This is called before CheckIn to get from
        the server the source files that are currently checked out and downloaded locally and how the system is going
        to search for translated files locally for CheckIn. The information here is going to be used to search
        additional files in the client.  If the ItemRevision business object is under Item Revision Definition
        Configuration (IRDC), the CheckIn mode value is retrieved from the IRDC object; otherwise it will be an empty
        string.
        CheckIn mode is used to check in translated files that are already in the directory with the source file or the
        first level subdirectory. 
        There are three CheckIn modes:
        - Same File Name: Attaches and checks in the derived files only if they have the same name as the source
        dataset.
        - Any File Name: Attaches and checks in the derived files no matter what names they have.
        - None: Does not attach and check in any derived files.
        
        
        Refers to Business Modeler IDE Guide > Creating data model objects to represent objects in Teamcenter > Working
        with document management > Create an Item Revision definition configuration (IRDC) for more information, the
        Checkin mode is defined by Derived Visualization Files to Checkin from the IRDC Checkin Page Info tab.
        
        Use cases:
        Check in ItemRevision under IRDC control
        When a user checks out an ItemRevision business object under IRDC control, the user has the option to download
        the source files into user local machine.  If the user then checks in the ItemRevision, the system will search
        for the translated files in the source file directory according to the specified CheckIn mode. This
        functionality supports client side rendering to provide the derived datasets.
        For example, there is case where some AutoCAD file cannot be converted to a certain format by the server; user
        can find the translated files in the local directory to check in instead.
        """
        return cls.execute_soa_method(
            method_name='getCheckinModeAndFiles',
            library='DocumentManagement',
            service_date='2008_06',
            service_name='DocumentControl',
            params={'inputs': inputs},
            response_cls=GetCheckinModeAndFilesOutputsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.ImportExport._2011_06.FileImportExport import ImportExportOptions
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportDocumentInputData(TcBaseObj):
    """
    Structure represents the parameters required to import a requirement specification.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var transientFileWriteTicket: Transient File write ticket for the file to be imported.
    :var childSpecElementType: Type of child elements to be created on import.
    :var specificationType: Type of top level object to be created. RequirementSpec is default.
    :var selectedObject: Object under which new structure will be imported.
    :var keywordImportOptions: List of options to be used during keyword import.
    :var specDesc: Description to be set for an Specification.
    """
    clientId: str = ''
    transientFileWriteTicket: str = ''
    childSpecElementType: str = ''
    specificationType: str = ''
    selectedObject: BusinessObject = None
    keywordImportOptions: KeywordImportOptions = None
    specDesc: str = ''


@dataclass
class KeywordImporCondition(TcBaseObj):
    """
    Structure contains the operation type and keyword value provided by the user.
    
    :var opType: An operation type which is  to be applied on the document together with the values.
    Suppored operation types are :
    WORD_PARTIAL_MATCH : Searches for the partial occurrence of the given string.
    WORD_EXACT_MATCH : Searches for the exact occurrence of the given string.
    HAS_STYLE : Searches for the text with given style.
    :var keyword: The keyword here means the text to be searched for in the MS word document, is 
    provided by user to be used with operation type for Keyword Import keyword.
    Ex.  if the opType is WORD_EXACT_MATCH and value is "car" then the exact occurrence of "car" has to be looked up in
    the document .
    """
    opType: str = ''
    keyword: str = ''


@dataclass
class KeywordImportOptions(TcBaseObj):
    """
    Structure to specify the options to be used during keyword import.
    
    :var importOptions: Generic options in the form of key-value pairs to be used for processing.
    Ex. PermanentConvertToHTML, TRUE
    This option is used to convert the word file data in to HTML format to get this edited in active workspace client.
    RunInBackGround, TRUE
    This option is used to run Import process in background.
    :var keywordImportRules: Keyword import rules to be used.
    """
    importOptions: List[ImportExportOptions] = ()
    keywordImportRules: List[KeywordImportRule] = ()


@dataclass
class KeywordImportRule(TcBaseObj):
    """
    Structure contains the Rules provided by user to be used during keyword Import.
    
    :var targetChildType: SpecElement type which is to be created if the keyword rule evaluates to true on the document
    text.
    :var conditionProcessingType: The condition processing type, Supported values are: ANY and ALL. ANY is default.
    ANY &ndash; if any of the condition are met then only apply this rule.
    ALL -  if all the conditions are met then only apply this rule.
    :var keywordImportConditions: The conditions to be applied on the document to determine the types of objects to be
    created.
    """
    targetChildType: str = ''
    conditionProcessingType: str = ''
    keywordImportConditions: List[KeywordImporCondition] = ()



from tcsoa.gen.ProjectManagement._2007_01.services import ScheduleManagementService as imp0
from tcsoa.gen.ProjectManagement._2008_06.services import ScheduleManagementService as imp1
from tcsoa.gen.ProjectManagement._2012_02.services import ScheduleManagementService as imp2
from tcsoa.gen.ProjectManagement._2012_09.services import ScheduleManagementService as imp3
from tcsoa.gen.ProjectManagement._2015_07.services import ScheduleManagementService as imp4
from tcsoa.gen.ProjectManagement._2007_06.services import ScheduleManagementService as imp5
from tcsoa.gen.ProjectManagement._2011_06.services import ScheduleManagementService as imp6
from tcsoa.gen.ProjectManagement._2011_12.services import ScheduleManagementService as imp7
from tcsoa.gen.ProjectManagement._2016_04.services import ScheduleManagementService as imp8
from tcsoa.gen.ProjectManagement._2014_10.services import ScheduleManagementService as imp9
from tcsoa.gen.ProjectManagement._2014_06.services import ScheduleManagementService as imp10
from tcsoa.gen.ProjectManagement._2007_12.services import ScheduleManagementService as imp11
from tcsoa.gen.ProjectManagement._2009_10.services import ScheduleManagementService as imp12
from tcsoa.gen.ProjectManagement._2011_02.services import ScheduleManagementService as imp13
from tcsoa.gen.ProjectManagement._2018_11.services import ScheduleManagementService as imp14
from tcsoa.gen.ProjectManagement._2017_11.services import ScheduleManagementService as imp15
from tcsoa.base import TcService


class ScheduleManagementService(TcService):
    addMemberships = imp0.addMemberships
    addMemberships2 = imp1.addMemberships
    assignResources = imp2.assignResources
    assignResources2 = imp3.assignResources
    assignResources3 = imp4.assignResources
    baselineTasks = imp0.baselineTasks
    claimAssignment = imp3.claimAssignment
    copySchedules = imp0.copySchedules
    copySchedules2 = imp5.copySchedules
    copySchedules3 = imp1.copySchedules
    copySchedules4 = imp6.copySchedules
    copySchedulesAsync = imp6.copySchedulesAsync
    copySchedulesAsyncClient = imp6.copySchedulesAsyncClient
    createBillRates = imp1.createBillRates
    createDependencies = imp7.createDependencies
    createDependencies2 = imp2.createDependencies
    createNewBaselines = imp0.createNewBaselines
    createNewBaselines2 = imp8.createNewBaselines
    createNewBaselinesAsync = imp8.createNewBaselinesAsync
    createOrUpdateNotificationRules = imp5.createOrUpdateNotificationRules
    createPhaseGateTask = imp9.createPhaseGateTask
    createProxyTasks = imp6.createProxyTasks
    createSchedule = imp0.createSchedule
    createSchedule2 = imp1.createSchedule
    createScheduleDeliverableTemplates = imp0.createScheduleDeliverableTemplates
    createScheduleDeliverableTemplates2 = imp1.createScheduleDeliverableTemplates
    createTaskDeliverableTemplates = imp0.createTaskDeliverableTemplates
    createTaskDeliverableTemplates2 = imp5.createTaskDeliverableTemplates
    createTasks = imp2.createTasks
    deleteAssignments = imp2.deleteAssignments
    deleteDependencies = imp2.deleteDependencies
    deleteNotificationRules = imp5.deleteNotificationRules
    deleteScheduleAsync = imp6.deleteScheduleAsync
    deleteScheduleAsyncClient = imp6.deleteScheduleAsyncClient
    deleteSchedulingObjects = imp0.deleteSchedulingObjects
    deleteTasks = imp2.deleteTasks
    detachSchedule = imp9.detachSchedule
    filterUsers = imp10.filterUsers
    findCriticalPathTasks = imp2.findCriticalPathTasks
    getCostRollupData = imp9.getCostRollupData
    getDemandProfile = imp11.getDemandProfile
    getEVMResults = imp9.getEVMResults
    getNotificationRules = imp5.getNotificationRules
    getResourceGraphData = imp9.getResourceGraphData
    insertSchedule = imp9.insertSchedule
    launchScheduledWorkflow = imp2.launchScheduledWorkflow
    loadSchedules = imp6.loadSchedules
    manageScheduleLocks = imp6.manageScheduleLocks
    modifySchedules = imp12.modifySchedules
    moveTasks = imp2.moveTasks
    recalculateScheduleNonInteractive = imp2.recalculateScheduleNonInteractive
    refreshScheduleObject = imp6.refreshScheduleObject
    replaceAssignment = imp9.replaceAssignment
    scaleScheduleNonInteractive = imp2.scaleScheduleNonInteractive
    shiftSchedule = imp4.shiftSchedule
    shiftScheduleAsync = imp4.shiftScheduleAsync
    shiftScheduleNonInteractive = imp2.shiftScheduleNonInteractive
    specialPasteScheduleTasks = imp13.specialPasteScheduleTasks
    submitTimesheetEntries = imp14.submitTimesheetEntries
    submitTimesheetEntriesAsync = imp14.submitTimesheetEntriesAsync
    updateAssignments = imp2.updateAssignments
    updateDependencies = imp2.updateDependencies
    updateSchedules = imp2.updateSchedules
    updateTaskCostData = imp1.updateTaskCostData
    updateTaskExecution = imp6.updateTaskExecution
    updateTasks = imp12.updateTasks
    updateTasks2 = imp2.updateTasks
    verifySchedule = imp9.verifySchedule
    whatIfAnalysis = imp15.whatIfAnalysis



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanRelation
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class MultiRelMultiLevelExpandInput(TcBaseObj):
    """
    Input structure for createRelations operation.
    
    :var object: This is the starting object.
    :var relationships: Relationships used in search.
    """
    object: BusinessObject = None
    relationships: List[MultiRelationMultiLevelExpandRelationship] = ()


@dataclass
class MultiRelationMultiLevelExpandChildNodes(TcBaseObj):
    """
    This structure contains information for the child nodes in the tree structure returned by the multi-level expand.
    
    :var relationship: The relationship.
    :var children: The children found.
    """
    relationship: MultiRelationMultiLevelExpandRelationship = None
    children: List[MultiRelationMultiLevelExpandOutputNode] = ()


@dataclass
class MultiRelationMultiLevelExpandOutputNode(TcBaseObj):
    """
    This contains the tree structure returned by the multi level expand. This starts with the root node. And contains
    all its children based on the input relationships.
    
    :var currentObject: Current object represented by this node.
    :var relation: Relation of current object and children.
    :var relAndChildNodes: A list of the expansion nodes beneath the object
    """
    currentObject: BusinessObject = None
    relation: ImanRelation = None
    relAndChildNodes: List[MultiRelationMultiLevelExpandChildNodes] = ()


@dataclass
class MultiRelationMultiLevelExpandRelationship(TcBaseObj):
    """
    This structure represents the relationship being used and if the input object is the primary or secondary object.
    
    :var relationName: Name of the relationship.
    :var isPrimaryObject: If the object in 'MultiRelMultiLevelExpandInput' is on the primary object in the relationship
    """
    relationName: str = ''
    isPrimaryObject: bool = False


@dataclass
class MultiRelationMultiLevelExpandResponse(TcBaseObj):
    """
    This contains the tree structure returned by the multi-level expand. The node, foundObjects, starts with the input
    node.  It contains all it's children based on the input relationships.
    
    :var serviceData: Service data
    :var foundObjects: Objects found
    """
    serviceData: ServiceData = None
    foundObjects: MultiRelationMultiLevelExpandOutputNode = None


@dataclass
class Relationship(TcBaseObj):
    """
    'Relationship' structure represents all required parameters to create the relation between the primary and
    secondary object.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this 'Relationship' structure.
    :var relationType: Name of the relation type to create, required. This could be an empty string, in which case the
    relation name will be searched in the preference, ParentTypeName_ChildTypeName_default_relation or
    ParentTypeName_default_relation.
    :var primaryObject: The primary object to create the relation from.
    :var secondaryObject: The secondary object to create the relation to.
    :var userData: The user data object used to create the relation. This parameter is optional.
    """
    clientId: str = ''
    relationType: str = ''
    primaryObject: BusinessObject = None
    secondaryObject: BusinessObject = None
    userData: BusinessObject = None


@dataclass
class CreateRelationsOutput(TcBaseObj):
    """
    This structure contains information for 'createRelations' operation.
    
    :var clientId: The unmodified value from the 'Relationship.clientId'. This can be used by the caller to indentify
    this data structure with the source input data.
    :var relation: The newly created relation.
    """
    clientId: str = ''
    relation: ImanRelation = None


@dataclass
class CreateRelationsResponse(TcBaseObj):
    """
    'CreateRelationsResponse' structure represents the relations created between the primary and secondary object and
    errors occurred.
    
    :var output: A list of created relations.
    :var serviceData: Standard 'ServiceData' object to hold the partial errors that the operation encounters.
    """
    output: List[CreateRelationsOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportUnmanagedDataResponse(TcBaseObj):
    """
    ImportUnmanagedDataResponse type of structure is return as response from this SOA operation.
    
    :var transientFileReadTickets: FMS Ticket of log files.
    :var modifiedObjectList: List of modified objects.
    :var serviceData: The Service Data.
    """
    transientFileReadTickets: List[str] = ()
    modifiedObjectList: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class RelativeStructureChildInfo(TcBaseObj):
    """
    'RelativeStructureChildInfo' structure holds information about child objects in a structure.
    RelativeStructureParentInfo holds list of structures of type RelativeStructureChildInfo.
    
    :var childID: Item Id for child object
    :var objectType: Type of the object e.g. Item, Part, Vendor, Part etc
    :var refDesignator: The reference designator value.
    :var childAttrList: List of attribute on child object
    :var attrListRel: The list of attributes on the relationship between parent and child.
    :var relationType: Type of relation PSOccurrence or GRM
    :var actionRequired: If the object is modified, New or Deleted from Teamcenter. If user passed as empty string then
    server will figure out the action to be trigger.
    :var structModification: The map to store id of the child element if it is added, replaced  or deleted.
    :var childChangeType: Possible value for this parameter are  "BVR","GRM","FORM","FOLDERS"  and "NO_CHANGE". If user
    passed as empty string then server will figure out the change type.
    """
    childID: str = ''
    objectType: str = ''
    refDesignator: str = ''
    childAttrList: AttrMap = None
    attrListRel: AttrMap = None
    relationType: str = ''
    actionRequired: str = ''
    structModification: AttrMap = None
    childChangeType: AttrMap = None


@dataclass
class RelativeStructureParentInfo(TcBaseObj):
    """
    'RelativeStructureParentInfo' structure holds information about parent object.
    
    :var parentID: ID of Parent.
    :var objectType: Business object type of parent.
    :var parentAttrList: Attribute map of parent object.
    :var childInfo: Vector that holds structure of type  RelativeStructureChildInfo.
    :var actionRequired: Possible value for this parameter are "New" or "Modify". If user passed as empty string then
    server will figure out the action to be trigger.
    :var parentChangeType: Possible value for this parameter are "BVR","GRM","FORM","FOLDERS" and "NO_CHANGE". If user
    passed as empty string then server will figure out the change type.
    """
    parentID: str = ''
    objectType: str = ''
    parentAttrList: AttrMap = None
    childInfo: List[RelativeStructureChildInfo] = ()
    actionRequired: str = ''
    parentChangeType: AttrMap = None


"""
This map will hold the name value pair being passed to SOA
"""
AttrMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class MakeUserResponse(TcBaseObj):
    """
    The return data for the makeUser service operation.
    
    :var commandStatus: The make_user command completion status (0 - Success, 1 - Failure). See the make_user utility
    documentation for details.
    :var standardOutputFmsTicket: FMS ticket for the make_user standard output. This element is empty if the
    enableStandardOutput parameter for the operation is false.
    :var standardErrorFmsTicket: FMS ticket for the make_user standard error. This element is empty if the
    enableStandardError parameter for the operation is false.
    :var serviceData: Any errors that occur transferring files to or from FMS or in launcing the make_user utility are
    included in the list of partial errors.
    """
    commandStatus: int = 0
    standardOutputFmsTicket: str = ''
    standardErrorFmsTicket: str = ''
    serviceData: ServiceData = None
